#version 150

#moj_import <fog.glsl>

in vec3 Position;
in vec2 UV0;
in vec4 Color;
in vec3 Normal;

uniform mat4 ModelViewMat;
uniform mat4 ProjMat;
uniform mat3 IViewRotMat;
uniform int FogShape;
uniform float GameTime;

out vec2 texCoord0;
out float vertexDistance;
out vec4 vertexColor;

void main() {
    vec4 pos = vec4(Position, 1.0);
    gl_Position = ProjMat * ModelViewMat * pos;

    texCoord0 = UV0;
    vertexDistance = fog_distance(pos.xyz, FogShape);

    vec4 col = Color;

    if (Normal.y == -1) {
        col.rgb *= 255.0 / 174.0;
    }
    else if (Normal.y == 1) {
        col.rgb *= 255.0 / 255.0;
    }
    else if (abs(Normal.x) == 1) {
        col.rgb *= 255.0 / 225.0;
    }
    else if (abs(Normal.z) == 1) {
        col.rgb *= 255.0 / 200.0;
    }

    float yval = 0.0;
    int faceVert = gl_VertexID % 4;
    if (((faceVert == 1 || faceVert == 2) && abs(Normal.x) == 1)
      ||((faceVert == 0 || faceVert == 1) && abs(Normal.z) == 1)){
        yval = 1.0;
    }

    if (Normal.y == 1) {
        yval = 1.0;
    }

    float morningness = length(col.rgb - vec3(255, 255, 255) / 255.0);
    morningness = smoothstep(0.08, 0.4, morningness); 
    float eveningness = length(col.rgb - vec3(133, 133, 141) / 255.0);
    eveningness = smoothstep(0.08, 0.6, eveningness);
    float nightness = length(col.rgb - vec3(25, 25, 38) / 255.0);
    nightness = smoothstep(0.02, 0.08, nightness);
    float thunderness = length(col.rgb - vec3(0, 0, 0) / 255.0);
    thunderness = smoothstep(0.2, 0.25, thunderness);

    vec3 dayCloud = mix((vec3(255, 255, 255) / 255), (vec3(207, 229, 247) / 255), 1 - yval);
    vec3 nightCloud = mix((vec3(10, 14, 36) / 255), (vec3(29, 42, 107) / 255), 0.4 * yval);
    vec3 eveningCloud = mix((vec3(89, 93, 115) / 255), (vec3(109, 113, 135) / 255), 0.4 * yval);
    vec3 thunderCloud = mix((vec3(40, 45, 63) / 255), (vec3(50, 55, 73) / 255), 0.4 * yval);
    vec3 thunderCloudNight = mix((vec3(7, 12, 33) / 255), (vec3(17, 22, 43) / 255), 0.4 * yval);

    vec3 currentCloudColor = mix(dayCloud, mix(eveningCloud, mix(nightCloud, mix(thunderCloudNight, thunderCloud, thunderness), nightness), eveningness), morningness);

    vertexColor = vec4(currentCloudColor.rgb, 0.8);
}
