#version 150

#moj_import <light.glsl>
#moj_import <fog.glsl>

in vec3 Position;
in vec4 Color;
in vec2 UV0;
in ivec2 UV2;
in vec3 Normal;

uniform sampler2D Sampler2;

uniform mat4 ModelViewMat;
uniform mat4 ProjMat;
uniform vec3 ChunkOffset;
uniform int FogShape;

out float vertexDistance;
out vec4 vertexColor;
out vec2 texCoord0;
out vec4 normal;

void main() {
    vec3 pos = Position + ChunkOffset;
    gl_Position = ProjMat * ModelViewMat * vec4(pos, 1.0);
    float skyLight = UV2.y / 16.0;
    float blockLight = UV2.x / 16.0;

    vec4 pureSkyColor = texture(Sampler2, clamp(vec2(0.0, 240.0) / 256.0, vec2(0.5 / 16.0), vec2(15.5 / 16.0)));
    vec4 defaultLightColor = texture(Sampler2, clamp(UV2 / 256.0, vec2(0.5 / 16.0), vec2(15.5 / 16.0)));

    float eveningness = length(pureSkyColor.xyz - vec3(142, 142, 176) / 255);
    eveningness = smoothstep(0.08, 0.4, eveningness);
    float nightness = length(pureSkyColor.xyz - vec3(39, 39, 64) / 255);
    nightness = smoothstep(0.29, 0.8, nightness);
    float rainingness = length(pureSkyColor.xyz - vec3(196, 196, 211) / 255);
    rainingness = smoothstep(0.01, 0.1, rainingness);
    float thunderness = length(pureSkyColor.xyz - vec3(163, 163, 190) / 255);
    thunderness = smoothstep(0.15, 0.2, thunderness);

    vec3 daySkyColor = vec3(81, 176, 255) / 255;
    vec3 nightSkyColor = vec3(5, 9, 26) / 255;
    vec3 eveningSkyColor = vec3(85, 89, 106) / 255;
    vec3 thunderSkyColor = vec3(35, 39, 56) / 255;

    vec3 currentSkyColor = mix(nightSkyColor, mix(eveningSkyColor, Color.rgb + daySkyColor * 0.2, eveningness), nightness);
    currentSkyColor = mix(eveningSkyColor, currentSkyColor, rainingness);
    currentSkyColor = mix(thunderSkyColor, currentSkyColor, thunderness);

    vec3 nightBlockLight = mix(currentSkyColor, vec3(0.5, 0.3, 0.2), smoothstep(0.0, 16.0, blockLight));
    vec3 dayBlockLight = mix(currentSkyColor, currentSkyColor * 1.2, smoothstep(0.0, 16.0, blockLight));

    if(blockLight > 0.0){
        vertexColor = vec4(mix(nightBlockLight, mix(nightBlockLight, mix(nightBlockLight, dayBlockLight, nightness), thunderness), rainingness), Color.a);
    }else if(skyLight > 0.0 && skyLight < 14.0){
        vertexColor = vec4(mix(currentSkyColor, currentSkyColor * minecraft_sample_lightmap(Sampler2, UV2).rgb, smoothstep(16.0, 10.0, skyLight)), Color.a);
    }else{
        vertexColor = vec4(currentSkyColor, Color.a);
    }
    
    if(skyLight == 0.0){
        vertexColor = Color * minecraft_sample_lightmap(Sampler2, UV2);
    }

    vertexDistance = fog_distance(pos, FogShape);
    texCoord0 = UV0;
    normal = ProjMat * ModelViewMat * vec4(Normal, 0.0);
}
